#ifndef INPUTINC
#define INPUTINC

#include <list>
#include <string>

#include "behavior.h"

enum InputTypes { AxisInput, ButtonInput };
enum AxisStates { AxisRelative, AxisAbsolute };

class gs_InputDevice;
class gs_InputData;

struct InputState
{
	gs_InputDevice* Parent;
	unsigned int Time;	// time in milliseconds
	int Index;			// which value (x,y or z axis, a,b,or c key)
	int Data;			// actual value
};
typedef InputState gs_InputState;

// this listens for any input events
// message is passed as an inputstate struct
class gs_InputListener
{
public:
	// see more notes under gs_InputDevice::AddNotify(..)
	virtual int Message(gs_InputState* input) { return 0; }
};

class gs_InputManager : public gs_Behavior
{
public:
	// retrieves individual devices
	// such as mouse, keyboard, or joystick
	virtual int GetTotal() { return 0; }
	virtual gs_InputDevice* Get(int x) { return 0; }
	virtual gs_InputDevice* Get( char* name) { return 0; }

	// pointers provide a way to bind to a specific data item
	// isValid is a way of checking if that pointer is still valid
	virtual int isValid( gs_InputDevice* ref) { return 1; }

	// polls and updates all the input devices
	void Run();
};

////////////////////////////////////////////////
#include "binding.h"


class gs_InputDevice
{
public:
	virtual ~gs_InputDevice() { };

	// this returns input but does not update
	// the pointer it returns should be valid throughout
	// the life of the device
	virtual gs_InputData& GetInput() = 0;	
	virtual void Update() { };	// this updates the input
	virtual void Set(int index, int pos) { }
		
	// gets the index for a common device part
	// ex. a, x axis, joystick button 1
	// flags are defined in binding.h
	virtual int GetIndex( string name ) { return 0; }
	virtual string GetName( int index) { return ""; }

	// returns a binding that can retrieve data
	// type = value or state
	gs_Binding GetBinding( int index );

	// more friendly version
	gs_Binding GetBinding( string name);
	int Get( int index);
	int Get( string name);

	////////////////////////
	virtual int GetTotalAxes() { return 0;}
	virtual int GetTotalButtons() { return 0; }
	int GetTotalKeys() { return GetTotalButtons(); }
	virtual int GetTotalPOVs() { return 0; }

	virtual void SetInputBuffer(int size) { }
	virtual int AddNotify( gs_InputListener* b) { return 0; } 
	virtual int RemoveNotify( gs_InputListener* b) { return 0; } 
	// this calls a method with parameters 
	// total events
	// pointer to event list
	// the overhead is acceptable because a function would need
	// to be called to store the input anyhow

	virtual void SetAxisMode( AxisStates state ) { }
	virtual string GetType() { return "unknown";}	// get device type (ie mouse, keyboard, joystick, ff joystick)
};

class gs_InputData
{
public:
	// this needs to be set at the beginning of the lifetime of
	// this object
	// it is not a constructor because then it would be more
	// difficult to include as a non-pointer member
	void Set( gs_InputDevice *parent, int *v, int tv, unsigned char* s, int ts)
	{ 
		Parent = parent;
		ValueData = v;
		TotalValues = tv;
		StateData = s;
		TotalStates = ts;
	}
	// different types of data
	// axes, pov switch
	// button states

	// value is a variable of any amount that indicates 
	// a device position (slider, axis, etc)
	// macros can be used to offer lexical names for values
	// ex. KEY_A = scancode of a = index for getvalue on keyboard
	inline const int GetValue( int index) { return ValueData[index]; }

	// state is any on/off switch (keyboard, buttons, etc)
	inline const unsigned char GetState( int index) { return StateData[index]; }

	inline const int Get( int index) 
	{
		// REMEMBER DO BOUNDS CHECKING
		if (index < TotalValues)
			return ValueData[index];
		else
			return StateData[index-TotalValues];
	}

	inline int GetTotalValues() { return TotalValues; }
	inline int GetTotalStates() { return TotalStates; }
	inline int GetTotal() { return TotalValues + TotalStates; }

	inline gs_InputDevice* GetParent();
private:
	gs_InputDevice *Parent;
	int *ValueData;
	unsigned char *StateData;

	int TotalValues;
	int TotalStates;
};


#endif